//-------------------------------------------------------------------
//	File:		   CameraScript.cpp
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "camerascript.h"
#include "hardware.h"

//-------------------------------------------------------------------

CameraScript::CameraScript() {
	nitems=0; // # stored
	itemn=0;  // current frame 0->MAXSCRIPTENTRIES-1
	pos=0;    // current position in frame 0->1
	lastEntryTime=0;
	interval=1; // record interval
	record=0;
	play=0;
	denom=6;
	type=CATMULL;//BSPLINE,CATMULL // should add cos & lin & pos/orient types
};
CameraScript::~CameraScript() {};

//-------------------------------------------------------------------

void CameraScript::insert(Place *place, int force) { // force means insert now vs. timed
	if (nitems>=MAXSCRIPTENTRIES) { // table full?
		record=0; // stop recording
		return;
	}
	if ((force==0)&&((Hw::get()->time.time-lastEntryTime)<interval)) return; // time to insert?
	lastEntryTime=Hw::get()->time.time;
	Hw::get()->ann->hold("Script entry # %d",nitems);

	data[nitems].x=place->pos.x; // load table from controller values
	data[nitems].y=place->pos.y;
	data[nitems].z=place->pos.z;
	data[nitems].pitch  =place->orient.x;
	data[nitems].heading=place->orient.y;
	data[nitems].roll   =place->orient.z;
	nitems++;
}

//-------------------------------------------------------------------

int CameraScript::load(const char* infile) { // load file data to table
	char s[90];
	Place p;

	strcpy(s,infile);
	strcat(s,".cam"); // build filename to open
	ifstream file(s,ios::nocreate);
	if (file.bad()||(file==0)) return 1;

	nitems=0; // clear
	itemn=0;
	pos=0;
	file.getline(s,80); // skip header
	while (file.eof()==0) {
		file.getline(s,80); // read line

		if (s[0]==0) continue; // empty line?
		p.pos.x   =atof(strtok(s,   ", ")); // load tmp controller w/ data
		p.pos.y   =atof(strtok(NULL,", "));
		p.pos.z   =atof(strtok(NULL,", "));
		p.orient.x=atof(strtok(NULL,", "));
		p.orient.y=atof(strtok(NULL,", "));
		p.orient.z=atof(strtok(NULL," \n"));
		insert(&p,1); // load table w/ controller values (could load table directly)
	}
	file.close();
	return 0;
}

//-------------------------------------------------------------------

void CameraScript::script2place(Place *place){ // load controller from interp'd table
	if (nitems==0) return;
	int itemn2=(itemn +1)%nitems; // address next 3 nodes
	int itemn3=(itemn2+1)%nitems;
	int itemn4=(itemn3+1)%nitems;

	switch(type) {
		case BSPLINE: set_spline( pos); break;
		case CATMULL: set_catmull(pos); break;
	}
	place->pos.x   =get(data[itemn].x,      data[itemn2].x,      data[itemn3].x,      data[itemn4].x);
	place->pos.y   =get(data[itemn].y,      data[itemn2].y,      data[itemn3].y,      data[itemn4].y);
	place->pos.z   =get(data[itemn].z,      data[itemn2].z,      data[itemn3].z,      data[itemn4].z);
	place->orient.x=get(data[itemn].pitch,  data[itemn2].pitch,  data[itemn3].pitch,  data[itemn4].pitch);
	place->orient.y=get(data[itemn].heading,data[itemn2].heading,data[itemn3].heading,data[itemn4].heading);
	place->orient.z=get(data[itemn].roll,   data[itemn2].roll,   data[itemn3].roll,   data[itemn4].roll);
}

//-------------------------------------------------------------------

int CameraScript::save(const char* title) { // save table data to file
	char s[80];
	Place p;

	if (nitems!=0) {
		strcpy(s,title);
		strcat(s,".cam"); // build filename to open
		ofstream file(s,ios::out);
		if (file.bad()||(file==0)) return 1;

		file << "    x         y         z          pitch     heading   roll\n";
		for (int i=0; i<nitems; i++) {
			sprintf(s,"%9.3f %9.3f %9.3f   %7.1f %7.1f %7.1f\n",data[i].x,
																				 data[i].y,
																				 data[i].z, 
																				 data[i].pitch,
																				 data[i].heading,
																				 data[i].roll);
			file << s;
		}
		file.close();
	}
	return 0;
}

//-------------------------------------------------------------------

void CameraScript::update(float increment) { // move forward
	if (nitems!=0) {
		pos+=increment*Hw::get()->time.dt;
		while (pos>=1) { // on next frame?
			pos-=1;       // backup position
			itemn=(itemn+1)%nitems; // increment frame circularly
		}
	}
}

//-------------------------------------------------------------------

float CameraScript::get(float v1, float v2, float v3, float v4) { // interp 4 values
	return (a0*v1+a1*v2+a2*v3+a3*v4) / denom;
}

//-------------------------------------------------------------------

void CameraScript::set_spline(float pos) {
	float u1,u2,u3;

	u1=pos;
	u2=u1*u1;
	u3=u1*u2;
	denom=6;
	a0=-1 *u3 +3 *u2 -3 *u1+1;
	a1= 3 *u3 -6 *u2 +0 *u1+4;
	a2=-3 *u3 +3 *u2 +3 *u1+1;
	a3= 1 *u3 +0 *u2 +0 *u1+0;
}

//-------------------------------------------------------------------

void CameraScript::set_catmull(float pos) {
	float u1,u2,u3;

	u1=pos;
	u2=u1*u1;
	u3=u1*u2;
	denom=2;
	a0=-1 *u3 +2 *u2 -1 *u1+0;
	a1= 3 *u3 -5 *u2 +0 *u1+2;
	a2=-3 *u3 +4 *u2 +1 *u1+0;
	a3= 1 *u3 -1 *u2 +0 *u1+0;
}

//-------------------------------------------------------------------

