//
// main.cpp
//
//  I use what some call a crazy file structure, since I "include" code along the way.
//  These include files aren't classes, so they are not just a separate cpp.
//  They also don't change very often, so I don't want to see them.
//  I generally only want to work in 1 file (and support cpps), "effect.cpp" and its header.
//  I occasionally may make tweaks to the framework, in "main.cpp",
//    as I relegate some new tested effect to permanent placement in the framework,
//    so a new graphic or control effect can start in effect.cpp and migrate to main and possibly std.
//  I am continuously playing with class design, somewhat new to c++ & OOP,
//    and I am hopefully iterating towards a more optimal file/object structure,
//    that meets my specific requirements of separating effect from framework.
//
//
//  file structure:
//
//    main.cpp:   basic minimal common framework stuff that changes slightly
//                testbed for new standard features
//                renders only built-in skybox/star/particle environment (key 'u' Universe)
//                1 generic light (key 's' Spotlight)
//
//    effect.cpp: all code specific to "this" graphic, in class form
//                should be capable of being defined out
//                changes app to app
//
//
//    included files (in main.cpp):
//
//      std.cpp   has rarely-changing established framework functions, callbacks, etc.
//                new testbed functions moved here when somewhat matured
//
//      dialog_main.cpp: 
//                "this app" gui menu definition and processing
//                legacy gui code in need of rewrite, my 1st major OOP class design,
//                and new additional features were kluged in and need proper incorporation.
//
//
//    misc:
//
//    hardware.cpp refers to a singleton global containing all general "hardware" objects:
//                 screen class w/ multiple viewports w/ independent cameras 
//                 mouse, key, time, userPrefs and the FMOD sound system
//                 also has a pointer "ann", to the standard onscreen-feedback text-annotation object
//                    which displays in the upper left corner for 3 seconds using "ann->hold("Hello");"
//
//    the rest are misc class files (text/texture/skybox/splinter/particles/mesh...)
//
//------------------------------------------------------------------------------

#define haseffect

#include "main.h"

#include "billboard.h"
#include "camerascript.h"
#include "config.h"
#include "dialog.h"
#include "fog.h"
#include "light.h"
#include "listbox.h"
#include "physics.h"
#include "random.h"
#include "skybox.h"
#include "starfield.h"
#include "text.h"
#include "texture.h"
//#include "histogram.h"

//------------------------ new global framework objects

Billboard sunBB;   // sun/moon/spotlight @ light0
CameraScript camscript;
//Histogram histogram;
Texture font2T;

#if defined haseffect
#include "effect.h"
Effect effect; // main "this app" effect class
#endif

char s[80]={"FellowshipZ"}; // generic string

int demoend=1;

PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB=NULL;
PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB=NULL;
//PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB=NULL;

void do_sound(void);
void draw_rect(float x, float y, float sizex, float sizey, float r, float g, float b);

//--------------------------------------------------------------------

#include "std.cpp"  // all standard rarely-changing stuff

//--------------------------------------------------------------------

void main(int argc, char** argv) {
	hw=Hw::get(); // singleton global w/ all hardware and prefs
	hw->ann=&ann; // reference standard annotation

	strcpy(hw->title,s);     // set app name
	strcpy(s,"Starting up ");// make log entry
	strcat(s,hw->title);
	strcat(s,"...");
	log_message(s);

	hw->pref.set(hw->title); // load prefs from file or hardcoded
	config.load(hw->title);  // overide w/ config

	glut_init(argc, argv);
}

//--------------------------------------------------------------------

#include "dialog_main.cpp" // old dialog system

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

void annot_this(void){ // 2d ortho draw in current viewport over 3d stuff
	//sprintf(s,"%d %d %d",hw->key.alt,hw->key.control,hw->key.shift);
	//ann.hold(s);

	/*
	glEnable(GL_TEXTURE_2D);
	glColor3f(1,1,1);
	effect.arcT.bind();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(0,0);
		glTexCoord2f(1,0); glVertex2f(256,0);
		glTexCoord2f(1,1); glVertex2f(256,256);
		glTexCoord2f(0,1); glVertex2f(0,256);
		glEnd();

	dt.bind();
	glEnable(GL_BLEND);
	set_blend(0,6);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(0,0);
		glTexCoord2f(2,0); glVertex2f(256,0);
		glTexCoord2f(2,2); glVertex2f(256,256);
		glTexCoord2f(0,2); glVertex2f(0,256);
		glEnd();
glDisable(GL_BLEND);
*/
glDisable(GL_TEXTURE_2D);

	//histogram.draw();
/*

	set_blend(1,0);
	glEnable(GL_BLEND);
	float v,n=380, dotsize=2;
	static int x=0;
	for (int i=0; i<n; i++) {
		v=effect.perlin.get1(i,hw->time.time*40); // get perlin value for current time at channel i
		draw_rect(x,i*dotsize//+v*12
			,dotsize,dotsize, v,v,v);
	}
	x+=1;
	if (x>1000) x=0;
	glDisable(GL_BLEND);
*/

#if defined haseffect
	effect.annot(&ann);
#endif
}

//--------------------------------------------------------------------

void close_this(void) { // 1 time final shutdown
	log_message("Close this...");
	starfield.destroy();

#if defined haseffect
	effect.close();
#endif

	dialog->clear();
}

//--------------------------------------------------------------------

void draw_this(void){ // main 3d perspective draw in current viewport
	if (hw->pref.lights) {
		light.set_xyz(sinf(hw->time.time*.05)*hw->worldSize*1.5,
			           90,//hw->worldSize*.9,
						  cosf(hw->time.time*.05)*hw->worldSize*1.5);
		light.amb=hw->pref.ambient;
		light.diff=hw->pref.diffuse;
		light.refl=hw->pref.reflective;
		light.spec=hw->pref.specular;
		light.exponent=hw->pref.exponent;
		light.cutoff=hw->pref.cutoff;
		glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,hw->pref.shininess); // 5 bigspot-50 smallspot
		light.setup();
	}
	if (hw->pref.universe<4) { // render?
		sunBB.vis.set(); // set renderstate for sun
		if ((effect.effectn==1)&&(effect.nite==1)) sunBB.ta.image=23;
		else                                       sunBB.ta.image=2;/////////////////////
		sunBB.ta.set(0); // set tex
		glPushMatrix();
			if (effect.effectn!=3) glTranslatef(light.x,light.y,light.z);//////////////////
			else                   glTranslatef(      0,1000,0);
			sunBB.draw(1);
			glPopMatrix();
		sunBB.vis.unset();// unset render state
		sunBB.ta.unset(0);// unset tex state
	}

#if defined haseffect
	effect.draw(&ann);
#endif
}

//--------------------------------------------------------------------

void init_this(void) { // 1 time init
	float meter;

	log_message("Init this...");

	hw->worldSize=1200; // world spec -----------------------------
	meter=hw->meter=1;
	hw->screen.nearz=.05;//.01
	hw->screen.farz=hw->worldSize*3;//3.57;
	hw->screen.set_nviewports(hw->pref.viewportset); // set 1 viewport

	// main light ----------------
	light.set_params(hw->pref.ambient,hw->pref.diffuse,hw->pref.specular,hw->pref.reflective);
	light.exponent  =hw->pref.exponent;
	light.cutoff    =hw->pref.cutoff;

	sunBB.sizex=.025*hw->worldSize; // sun billboard -------------------------
	sunBB.sizey=sunBB.sizex;
	sunBB.vis.reset();
	sunBB.vis.fog=0;
	sunBB.vis.lights=0;
	sunBB.vis.alphatest=1; 
	sunBB.vis.alphafunc=.01; 
	sunBB.ta.image=2;
	sunBB.ta.textures=1;

	if (sunT.load_tex("images/sun_128.bmp", GL_RGBA, -1, 0, 0)) log_message("Texture sun load problem **********");
	//if (dt. make_tex(SAND,128,128,GL_LUMINANCE_ALPHA,1)) log_message("Cant make sand texture **********");
	//if (ct.load_tex("images/cloudbw2128.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0))  log_message("Cant make cloud texture **********");

	if (maxuniverse>3) {
		skybox.gap=1.5; // 50% > WorldSize
		skybox.init();  // load textures
	}

	log_message("Init starfield..."); // static displaylist far starfield  ------
	starfield.n=hw->pref.nstars;
	starfield.nlevel=2;
	starfield.size=hw->worldSize*2;
	starfield.hollow=.25*starfield.size;
	starfield.blend=1; starfield.depthtest=0; starfield.depthwrite=0;
	starfield.type=1; // 0-round/1-cube/2-dome
	starfield.init();// init volume univ

	log_message("Init infinite particles..."); // brownian floating particles -----
	starfield2.n=0;
	starfield2.size=hw->worldSize*2;
	starfield2.hollow=0*starfield2.size;
	starfield2.blend=1; starfield2.blendFunc0=2; starfield2.blendFunc1=3;
	starfield2.depthtest=1; starfield2.depthwrite=0;
	starfield2.type=1; // 0-round/1-cube/2-dome
	starfield2.tex=&sphereT;
	starfield2.sizei=1*meter;
	starfield2.init();// init volume univ

	log_message("Init std textures..."); // textures ---------------

	Text::charInX=13; Text::charInY=8; Text::tex=&fontT; // setup font
	fontT.repeatMode=GL_CLAMP;
	font2T.repeatMode=GL_CLAMP;
	if (fontT.load_tex("images/lucidaconsole.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0)) {
		log_message("Texture font2 load problem **********");
		glut_shutdown();
	}
	if (font2T.load_tex("images/tengwar_gandalf.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0)) {
		log_message("Texture font2 load problem **********");
		glut_shutdown();
	}

	if (sphereT.make_tex(SPHERE,128,128, GL_LUMINANCE_ALPHA,1)) log_message("Cant make sphere texture **********");
	//if (t1.load_tex("images/cloudbw2128.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0))  log_message("Cant make cloud texture **********");

	//if (smokeT.make_tex(SMOKE,  128,128,GL_LUMINANCE_ALPHA,1)) log_message("Cant make smoke texture **********");
	//if (starT.make_tex(STAR,    128,128,GL_LUMINANCE_ALPHA,1)) log_message("Cant make star texture **********");

	if (controlsT.load_tex("images/controls_128.bmp", GL_RGBA, -1, 0, 0)) log_message("Texture controls load problem **********");
	Widget::tex=&controlsT; // ----------- dialog
	Widget::text=&dialogText;
	dialogText.alphafunc=.05;
	dialogText.blend=0;
	dialog->text=&dialogText;
	dialog_init(MAIN,1);
	hw->dialogs=0;

	hw->screen.splinter.c2=.2;  // 20% fade effect
	hw->screen.splinter.plus=0; // only draw splinter (vs. plus current frame)

	camscript.load(hw->title);

	//histogram.nbins=80;
	//histogram.init();

#if defined haseffect
	effect.sphereT=&sphereT;
	effect.init(&ann);
#endif
}

//--------------------------------------------------------------------
	
void key_this(void){ // non-standard input
	static int reverbMode=0;

	if ((hw->skipSplash==0)||(hw->help==1)) hw->pref.page=1;// page used for scrolling

	if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.key!=0)) {
		switch(hw->key.key){
			case '8': reverbMode=(reverbMode==0)?25:reverbMode-1; hw->sound.set_reverb(reverbMode,&ann); break;
			case '9': reverbMode=(reverbMode>=25)?0:reverbMode+1;	hw->sound.set_reverb(reverbMode,&ann);	break;

			case 'a': scroller.msg("Hello"); break;

			case 'e': case 'E': dialog->refresh=2; break;

			case 'D': hw->pref.demosequence=!hw->pref.demosequence; ann.hold_multi(hw->pref.demosequence,"Demo Off","Demo On"); break;

#if defined haseffect
#endif
			case '<': case '>':
				//ann.hold("Tesselation n: %d", hw->pref.tess);
				ann.hold("Tesselation factor: %.3f", hw->pref.tessf);
				break;

			case 'z': hw->screen.set_dim(DIMA, 2, 1,0,0,1); break; // fade to red and back over 2 sec

			case '#':
				if (config.load(hw->title)) ann.hold("Can't load config.");
				else {
					ann.hold("Loaded prefs from config.");
					dialog->off();
					hw->help=0;
				}
				break;
		}
	}

	if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.shift==0)) { // plain special keys
		switch(hw->key.skey){
			case GLUT_KEY_PAGE_UP: case GLUT_KEY_PAGE_DOWN:
				if ((hw->skipSplash)&&(hw->help==0)) ann.hold("Page: %.3f", hw->pref.page);
				break;
			case GLUT_KEY_HOME: case GLUT_KEY_END:
				if ((hw->skipSplash)&&(hw->help==0)) ann.hold("Home: %.3f", hw->pref.home);
				break;
		}
	}

#if defined haseffect
	effect.key(&ann);
#endif

	if ((hw->key.key!=0)&&(dialog->refresh<1)) dialog->refresh=1; // update dialog
}

//--------------------------------------------------------------------

void reset_this(void){
	log_message("Reset this...");
	hw->screen.viewport->camera.reset();

#if defined haseffect
	effect.reset(&ann);
#endif
}

//--------------------------------------------------------------------

void splash_this(void){ // splashscreen
	float inten=.5-(fabs(frac(hw->time.time)-.5))+.5, y;  // .5->1->.5 loop

	splash.autoset=0; // manual setup, i.e. print call doesn't set and reset render state
	splash.set_xyz(.05*hw->screen.width,.8*hw->screen.height);
	splash.yinc=-2.25; // autostep down 2.5 char heights

	splash.bend=1;  // animated title, bend quad corners
	splash.bloom=1; // bloom char size
	splash.set_color(inten,inten,1-inten);
	splash.size=hw->screen.width*.035;//36
	splash.set();   // set render state
	splash.print(hw->title);
	splash.bend=0;
	splash.bloom=0;

	splash.set_color(1,.95,.9); // subtitle
	splash.size=hw->screen.width*.015;//15;
	splash.y+=10;
	splash.set();
	splash.print("a NeHe 'Lord of the Rings' contest entry");

	splash.y+=5; // copyright
	splash.size=hw->screen.width*.008;//8;
	splash.print("Copyright (C) 2002 by George Ziniewicz - zintel.com");
	y=splash.y;

	splash.size=hw->screen.width*.0098; // listbox instructions //10
	splash.y=.2*hw->screen.height;
	if (hw->skipSplash==0) {
		splash.print("<Left-Click> to start manual camera, <Middle-Click> to start demo...");
		splash.print("Or press <UP/DOWN ARROW> & <PAGE UP/DOWN> to read more.");
	} else {
		splash.y+=splash.yinc;
		splash.print("Press <UP/DOWN ARROW> & <PAGE UP/DOWN> to scroll.");
	}
	splash.unset();

	listbox.x=splash.x; // scrollable listbox view of info.txt
	listbox.y=y-hw->screen.height*.026;//20
	listbox.size=hw->screen.width*.0098;//10
	//listbox.yinc=hw->screen.height*-.025;//19
	if (hw->screen.height>700) listbox.yinc=-19;
	else                       listbox.yinc=-15;
	listbox.draw();
}

//--------------------------------------------------------------------

void start_this(void){ // every frame, prior to draw
	static int logod=0, hintn=0;
	int nhints;

	//histogram.insert(rndnn(1));

	if ((logod==0)&&(hw->time.time>8)) { // if i havent mentioned it yet and its time 2
		//if (hw->skipSplash)
		if (demoend) scroller.msg("Press anything to abort demo-ending...");
		logod=1;
	}
	if (camscript.record) camscript.insert(&hw->screen.viewport->camera.place,0); 
	if ((demoend=1)&&(hw->anyinput>0)) demoend=0;
	if (hw->anyinput==1) {
		hw->anyinput=2;
		ann.hold("Demo-end terminated.");
	}
	
	if (hw->skipSplash==0) { // havent switched yet from splashscreen lookat mode?
		if (hw->mouse.button[0]) { // switch from lookat to manual camera?
			hw->screen.viewport->camera.place.shakeampl=0;
			hw->skipSplash=1;
			hw->screen.viewport->camera.place.movemode=MANUAL; // manual mode
			ann.hold("Entering manual camera mode: mouseLook + Arrow/Click move");
			hw->sound.play2d("sounds/notify8.wav");
			hw->sound.set_volume(.3);
		}
		if (hw->mouse.button[1]) { // turn off splash?
			hw->skipSplash=1;
			ann.hold("Use 'v' to change ViewCam mode, or 'r' reset to start demo");
			hw->sound.play2d("sounds/notify8.wav");
			hw->sound.set_volume(.3);
		}
	}
	if (!hw->pref.user) {             // random changes allowed?
		int ret=randoms.update();      // do random stuff (screen skip/splinter/hyperspace)
		if ((ret==2)&&(dialog->refresh<1)) dialog->refresh=1; // need to update gui?
		if ((ret==1)&&(hw->screen.viewport->camera.place.movemode!=MANUAL)) { // rnd change Option?
			static int lastoption=hw->pref.option, lastimage=hw->pref.image;
			int option=-1, image=-1;

			while ((option==-1)||(option==lastoption)) option=rndn(maxoption+1); // select new option
			lastoption=option;
			hw->pref.option=option;
		}
	}

	do_sound();

	// random scrolling help
	if (((hw->time.time-scroller.lastscrolltime)>hw->pref.scrollInterval)&&(hw->pref.scrollInfo)) { // time?
		scroller.lastscrolltime=hw->time.time;
		if (demoend==1) { // only LOTR info
			switch(hintn) {
				case 0: scroller.msg("FellowshipZ - Copyright (C) 2002 by George Ziniewicz - zintel.com"); break;
				case 1: scroller.msg("Music: One Eye Dream, by Joe Hillman"); break;
				case 2: scroller.msg("Most textures by: David Gurrea Hernandez"); break;
				case 3: scroller.msg("'The Lord of the Rings' was written by John Ronald Reuel Tolkien (1892-1973)"); break;
				case 4: scroller.msg("Tolkien wrote the trilogy in England, starting in 1939 (the start of WW2), taking 12 years to complete."); break;
				case 5: scroller.msg("Some of Tolkien's earliest notes for LOTR were written in the frontline trenches of WW1."); break;
				//case 6: scroller.msg("Tolkien served as second lieutenant in the British Army, fought in the great battle of Somme, and returned shell-shocked and with trench-fever."); break;
				//case 6: scroller.msg("The languages Tolkien (himself a language professor) created for Middle Earth were greatly inspired by Finnish languages."); break;
				case 6: scroller.msg("The ring inscription: ash nazg durbatulk, ash nazg gimbatul, ash nazg thrakatulk agh burzum-ishi krimpatul"); break;
				case 7: scroller.msg("The ring inscription: One ring to rule them all, one ring to find them, one ring to bring them all, and in the darkness, bind them"); break;
				case 8: scroller.msg("Sh, it happens."); break;
				//case 9: scroller.msg("'The Lord of the Rings' film was shot in New Zealand, over 14 months, with a budget over $250 million."); break;
				//case 5: scroller.msg("Tolkien's son served in Europe in WW2."); break;
				//case 10: scroller.msg("Film for all three books were produced at the same time, for sequential yearly December release."); break;
				//case : scroller.msg("."); break;
			}
			nhints=9;
		} else {
			switch(hintn) {
				case 0: scroller.msg("Press 'c' to toggle FramePerSecond Clock."); break;
				case 1: scroller.msg("Press 'f' to toggle FullScreen/window."); break;
				case 2: scroller.msg("Press 'l' to select a Letterbox size 0-40% sideband."); break;
				case 3: scroller.msg("Press 'x' to toggle screen splinter effect."); break;
				case 4: scroller.msg("Press unshifted '<>' to adjust camera FOV/Zoom."); break;
				case 5: scroller.msg("Press 'u' to select a Universe type:starfield/particle/skybox combinations."); break;
				case 6: scroller.msg("Press 'v' to select a ViewCam mode: ManualArrow-MouseControl/LookAt.../LookFrom.../Orbit."); break;
				case 7: scroller.msg("Press 'w' to select WireFrame/Point/FilledPoly mode."); break;
				case 8: scroller.msg("Press 'r' to reset and randomize most params."); break;

				case 9: scroller.msg("Press '{}' to move camera/eye in & out."); break;
				case 10: scroller.msg("Press unshifted '+-' to adjust camera rotate speed in Orbit & LookAt modes."); break;
				case 11: scroller.msg("Press shifted '+-' to adjust camera move speed in Orbit & LookAt modes."); break;
				case 12: scroller.msg("Press '?' to view/scroll the info.txt file."); break;

				case 13: scroller.msg("Press 'C' to select ColorMapping: RGB,Sepia,Blue-Green/Green/InverseColor."); break;
				case 14: scroller.msg("Press 'd' to toggle DepthCue Fog (F5-F8 adjusts distances)."); break;
				case 15: scroller.msg("Press F5/F6 & F7/F8 to adjust near & far fog distance."); break;
				case 16: scroller.msg("Press 'I' to toggle this scrolling info."); break;
				case 17: scroller.msg("Press 'L' to toggle fullscreen lightmap (<Shift>F1-F10 for lightmap adjusters)."); break;
				case 18: scroller.msg("Press 'n' to select a numeric info item, like camera position, render rate, sounds playing...."); break;
				case 19: scroller.msg("Press 'U' to toggle User mode: no computer-randomized changes."); break;
				case 20: scroller.msg("Click Left mouse button for forward thrust, double click for 100x spurt in manual camera mode."); break;
				case 21: scroller.msg("Click Middle mouse button for reverse thrust/brake in manual camera mode."); break;
				case 22: scroller.msg("Click Right mouse button to toggle popup menus (and pause camera movement in manual camera mode.)"); break;
				case 23: scroller.msg("Note that an extensive log file is created for the current app run."); break;
				case 24: scroller.msg("Cheatcode for night-vision: rCCC (3 C's)"); break;
				case 25: scroller.msg("Cheatcode for god-view: press and hold }."); break;
			}
			nhints=26;
		}
		hintn=(hintn>=(nhints-1))?0:hintn+1;
	}

/*
			case 9: scroller.msg("Press 'e' to select an object for editing (then F1-F4 texmag adjust, b/B Blending, i Image textures, and position/orient if camera not active)."); break;
			case 5: scroller.msg("Press 'i' to select an Image option (texture) for selected object 'e'."); break;
			case 0: scroller.msg("Press 'b/B' to select BlendFunc0/BlendFunc1 for selected edit object 'e'."); break;
			case 28: scroller.msg("Press F1/F2 & F3/F4 to adjust texture X & Y mag for selected object ('e')."); break;
			case 30: scroller.msg("ArrowKeys move camera/selected edit object in look direction in x/z, <Control>+Arrow for x/y, <Shift>+Arrow for 10x thrust (manual camera/edit object mode ('e'))."); break;
			case 31: scroller.msg("Mouse changes camera/selected edit object orientation."); break;

			case 11: scroller.msg("Press 'o' to select an available Option."); break;
			case 12: scroller.msg("Press 'p' to toggle Pause Camera gravity effect."); break;
			case 17: scroller.msg("Press 'V' to select a viewport for camera control (1-6 to select a viewport set)."); break;
			case 27: scroller.msg("Press <Delete> to Jump pure vertical in manual camera mode."); break;
			case 35: scroller.msg("LightMap adjusters, all <Shift>ed : F1/F2-texMagX  F3/F4-texMagY  F5/F6-updateInterval  F7/F8-noiseType  F9/F10-noiseLevel  F11/F12-vignetteSize"); break;
			//case 26: scroller.msg("Press <PageUp>/<PageDown> to adjust ground height."); break;
			//case 22: scroller.msg("Press shifted '<>' to adjust tesselation."); break;
*/

	starfield2.brownian(); // jiggle virtual infinite particles
/*
	if (hw->pref.universe<4) fog.set_color(0,0,0);
	else                     fog.set_color(.05,.025,0);
*/

#if defined haseffect
	effect.start_frame(&ann);
#else
	if (hw->screen.viewport->camera.place.physics!=0) {
		hw->screen.viewport->camera.place.physics->rotatedrag.set_xyz(5,5,5); // nominal spin-damping
		if ((hw->help==0)&&(hw->dialogs==0))
			hw->screen.viewport->camera.place.physics->damp(&hw->screen.viewport->camera.place);
	}
#endif
}

//--------------------------------------------------------------------

void end_this(void){ // every frame after all viewports drawn
	if ((dialog->ID==GENERAL)&&(dialog->activeID==999)) { // gui reset?
		hw->pref.reset();
		reset_this();
		ann.hold("Reset to default prefs");
		dialog->off();
		if (dialog->refresh<1) dialog->refresh=1;
		dialog->activeID=-1;
	}
	if ((dialog->ID==GENERAL)&&(dialog->activeID==102)) { // hints toggle?
		scroller.lastscrolltime=0;
		dialog->activeID=-1;
	}

#if defined haseffect
	if ((dialog->ID==OBJECT)&&(dialog->activeID==39)) { // chg image?
		hw->pref.image=dialog->activeValue;
		if (effect.dispman.selObject>0)
			effect.dispman.dos[effect.dispman.selObject-1].texattribs[hw->pref.seltexlayer].image=hw->pref.image;
		dialog->activeID=-1;
	}
	if (dialog->activeID==1000) { // edit obj chg?
		effect.dispman.selObject=dialog->activeValue;
		if (effect.dispman.selObject>0) ann.hold(effect.dispman.dos[effect.dispman.selObject-1].label);
		else ann.hold("No object selected");
		effect.object2pref();
		dialog->refresh=2;
		dialog->activeID=-1;
	}

	if ((dialog->ID==THISAPP)&&(dialog->activeID==200)) { // chg scene?
		effect.proc_effect(2);
		effect.effectn=(effect.effectn==effect.neffects-1)? 0:effect.effectn+1;
		effect.proc_effect(0);
		dialog->activeID=-1;
	}

	effect.end_frame(&ann);
#endif
}

//--------------------------------------------------------------------

void do_sound(void) {
	static float lastSoundTime=0, interval=10;
	int i, isound, isounds[20];
	struct sound{
		char name[18];
		int freqrange,freqcenter, effectn;
	} sounds[]={
	// file[.wav]   frqrange  base   effectn
		"de11",     1000, 11000,   0,  // effect zones
		"gr11",       1000, 11000,   1,
		"whoosh2811",    4000,  7000,   2,
		"water_blup",    2000,  6000,   3,
		"28_57",         2000,  4000,   4,
		"ev11",          1000, 11000,   4,

		"wind2",         4000, 22000,   -1, // general sounds
		"owl1",          4000, 11000,   -1,
		"birds5",        2000, 11000,   -1,
		"birds6",        4000, 22000,   -1,
		"birds7",        4000, 22000,   -1,
		"crow",          2000, 11000,   -1,
		"wolf2",         4000, 22000,   -1,
		"cricket2",      6000, 22000,   -1,
	};
	int nsounds=14;

	if ( (hw->time.time-lastSoundTime)<interval) return; // not time yet?
	lastSoundTime=hw->time.time;
	interval=hw->pref.minSoundInterval+
		 rndn(hw->pref.maxSoundInterval-
			   hw->pref.minSoundInterval);
	isound=-1;
	for (i=0; i<nsounds; i++) {
		if ((sounds[i].effectn==effect.effectn)||(sounds[i].effectn==-1)) { // potential sound?
			isound++;
			isounds[isound]=i; // stash it
		}
	}
	isound++;
	if (isound>0) { // a sound found?
		isound=isounds[(int)(rndnn(2)*isound)]; // pick 1
		sprintf(s,"sounds/%s.wav",sounds[isound].name); // name.wav
		hw->sound.play(s);
		hw->sound.set_volume(hw->pref.ambientvolume);
		if (sounds[isound].freqrange) { // freq modifiable?
			hw->sound.set_frequency(rndn(sounds[isound].freqrange)-
			 .5*sounds[isound].freqrange+sounds[isound].freqcenter);
		}
	}
}

//------------------------------------------------------------------------

void after_swap(void) {
	effect.draw_eye();
}

//--------------------------------------------------------------------------
/*
void draw_rect(float x, float y, float sizex, float sizey, float r, float g, float b) {
	glColor3f(r,g,b);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(x      ,y);
		glTexCoord2f(1,0); glVertex2f(x+sizex,y);
		glTexCoord2f(1,1); glVertex2f(x+sizex,y+sizey);
		glTexCoord2f(0,1); glVertex2f(x      ,y+sizey);
		glEnd();
}
*/
//--------------------------------------------------------------------------


